<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\DataLayer;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator\Impression as ImpressionGa4;
use Flat101\EnhancedEcommerce\Model\Handler\ProductHandler;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\Catalog\Model\Product\Visibility as ProductVisibility;
use Magento\Store\Model\StoreManagerInterface;

class ProductRelatedView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'product-related-view-ga4';

    /** @var StoreManagerInterface */
    protected $storeManager;

    /** @var ImpressionGa4 */
    protected $impressionGeneratorGa4;

    /** @var Product */
    protected $productHandler;

    /** @var ProductRepositoryInterface */
    protected $productRepository;

    /** @var ProductVisibility */
    protected $catalogProductVisibility;

    /** @var Config */
    protected $configHelper;

    public function __construct(
        ImpressionGa4 $impressionGeneratorGa4,
        StoreManagerInterface $storeManager,
        ProductHandler $productHandler,
        ProductRepositoryInterface $productRepository,
        ProductVisibility $catalogProductVisibility,
        Config $configHelper
    ) {
        $this->storeManager = $storeManager;
        $this->impressionGeneratorGa4 = $impressionGeneratorGa4;
        $this->productHandler = $productHandler;
        $this->productRepository = $productRepository;
        $this->catalogProductVisibility = $catalogProductVisibility;
        $this->configHelper = $configHelper;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    /**
     * {@inheritdoc}
     */
    public function getLayer(): ?DataLayer\Dto\Ecommerce
    {
        /** @var ProductEntity $product */
        $product = $this->productHandler->getProduct();

        if (is_null($product)) {
            return null;
        }

        $impressionProducts = $this->getProductImpressions($product);

        $impression = new DataLayerGa4\Dto\Impression\Impression();
        $impression->currency = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $impression->items = $impressionProducts;
        $this->unset($impression, ['category']);

        $ecommerce = new DataLayer\Dto\Ecommerce();
        $ecommerce->event = 'view_item_list';
        $ecommerce->ecommerce = $impression;

        $this->unset($ecommerce, ['eventCategory','eventAction','eventLabel']);

        return $ecommerce;
    }

    public function getProductImpressions($product): ?array
    {
        // Get related Products with sort order
        $relatedProducts = $product->getRelatedProductCollection()
            ->addAttributeToSelect('required_options')->setPositionOrder()->addStoreFilter();
        // Set visibility
        $relatedProducts->setVisibility($this->catalogProductVisibility->getVisibleInCatalogIds());

        if (0 === \count($relatedProducts)) {
            return null;
        }

        $impressionProductsGa4 = [];
        $position = 0;
        $listLabel = $this->configHelper->getListLabelRelatedImpressions();

        /** @var Product $relatedProduct */
        foreach ($relatedProducts as $relatedProduct) {
            $entity = $this->productRepository->getById($relatedProduct->getId());
            $position++;

            $impressionProductsGa4[] = $this->impressionGeneratorGa4->generate($entity,
                $listLabel, ['position' => $position]);
        }

        // Unset product
        $this->productHandler->unsetProduct();

        return $impressionProductsGa4;
    }

    public function getImpressionsData(): ?array
    {
        /** @var ProductEntity $product */
        $product = $this->productHandler->getProduct();

        if (is_null($product)) {
            return null;
        }

        return $this->getProductImpressions($product);
    }
}
