<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model;

use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Magento\Framework\Session\SessionManager;

class Session extends SessionManager
{
    const KEY_ADD_PRODUCTS_TO_CART = 'add_products_to_cart';
    const KEY_REMOVED_PRODUCTS_FROM_CART = 'removed_products_from_cart';
    const KEY_CHECKOUT_STEPS = 'checkout_step';
    const KEY_CHECKOUT_OPTIONS = 'checkout_options';
    const KEY_IMPRESSION_CATALOG_PRODUCTS_UA = 'impression_catalog_products';
    const KEY_IMPRESSION_CATALOG_PRODUCTS_GA4 = 'impression_catalog_products_ga4';
    const KEY_IMPRESSION_SEARCH_PRODUCTS_UA = 'impression_search_products';
    const KEY_IMPRESSION_SEARCH_PRODUCTS_GA4 = 'impression_search_products_ga4';
    const KEY_CLICK_IMPRESSION_PRODUCTS_UA = 'click_impression_products';
    const KEY_CLICK_IMPRESSION_PRODUCTS_GA4 = 'click_impression_products_ga4';

    const KEY_PRODUCT_ADDITIONAL_INFO = 'product_additional_info';
    const KEY_BEHAVIORAL_EVENT = 'behavioral_events';

    public function setAddProductToCart($product): Session
    {
        $items = $this->getAddProductToCart();

        $this->storage->setData(self::KEY_ADD_PRODUCTS_TO_CART,
            array_merge($items, [$product]));

        return $this;
    }

    public function getAddProductToCart(bool $clear = true): array
    {
        $products = $this->getData(self::KEY_ADD_PRODUCTS_TO_CART, $clear);

        if (null === $products) {
            return [];
        }

        if (!is_array($products)) {
            return [];
        }

        return 0 < count($products) ? $products : [];
    }

    public function setRemovedProductFromCart($product): Session
    {
        $items = $this->getRemovedProductFromCart();

        $this->storage->setData(self::KEY_REMOVED_PRODUCTS_FROM_CART,
            array_merge($items, [$product]));

        return $this;
    }

    public function getRemovedProductFromCart(bool $clear = false): array
    {
        $products = $this->getData(self::KEY_REMOVED_PRODUCTS_FROM_CART,
            $clear);

        if (null === $products) {
            return [];
        }

        if (!is_array($products)) {
            return [];
        }

        return 0 < count($products) ? $products : [];
    }

    public function setCheckoutStep($paymentStepCheckout): Session
    {
        $steps = $this->getCheckoutSteps();

        $this->storage->setData(self::KEY_CHECKOUT_STEPS,
            array_merge($steps, [$paymentStepCheckout]));

        return $this;
    }

    public function getCheckoutSteps(bool $clear = false): array
    {
        $steps = $this->getData(self::KEY_CHECKOUT_STEPS, $clear);

        if (null === $steps) {
            return [];
        }

        if (!is_array($steps)) {
            return [];
        }

        return 0 < count($steps) ? $steps : [];
    }

    public function setImpressionCatalogProducts(
        $product,
        bool $uaDataLayerInsteadOfGa4 = false
    ): Session {
        $items = $this->getImpressionCatalogProducts($uaDataLayerInsteadOfGa4);

        $this->storage->setData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_IMPRESSION_CATALOG_PRODUCTS_UA
                : self::KEY_IMPRESSION_CATALOG_PRODUCTS_GA4,
            array_merge($items, [$product]));

        return $this;
    }

    public function getImpressionCatalogProducts(
        bool $uaDataLayerInsteadOfGa4 = false,
        bool $clear = false
    ): array {
        $items = $this->getData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_IMPRESSION_CATALOG_PRODUCTS_UA
                : self::KEY_IMPRESSION_CATALOG_PRODUCTS_GA4,
            $clear);

        if (!isset($items)) {
            return [];
        }

        if (!is_array($items)) {
            return [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setImpressionSearchProducts(
        $product,
        bool $uaDataLayerInsteadOfGa4 = false
    ): Session {
        $items = $this->getImpressionSearchProducts($uaDataLayerInsteadOfGa4);

        $this->storage->setData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_IMPRESSION_SEARCH_PRODUCTS_UA
                : self::KEY_IMPRESSION_SEARCH_PRODUCTS_GA4,
            array_merge($items, [$product]));

        return $this;
    }

    public function getImpressionSearchProducts(
        bool $uaDataLayerInsteadOfGa4 = false,
        bool $clear = false
    ): array {
        $items = $this->getData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_IMPRESSION_SEARCH_PRODUCTS_UA
                : self::KEY_IMPRESSION_SEARCH_PRODUCTS_GA4,
            $clear);

        if (!isset($items)) {
            return [];
        }

        if (!is_array($items)) {
            return [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setClickImpressionProducts(
        $product,
        bool $uaDataLayerInsteadOfGa4 = false
    ): Session {
        $items = $this->getClickImpressionProducts($uaDataLayerInsteadOfGa4);

        $this->storage->setData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_CLICK_IMPRESSION_PRODUCTS_UA
                : self::KEY_CLICK_IMPRESSION_PRODUCTS_GA4,
            array_merge($items, [$product]));

        return $this;
    }

    public function getClickImpressionProducts(
        bool $uaDataLayerInsteadOfGa4 = false,
        bool $clear = false
    ): array {
        $items = $this->getData(
            ($uaDataLayerInsteadOfGa4)
                ? self::KEY_CLICK_IMPRESSION_PRODUCTS_UA
                : self::KEY_CLICK_IMPRESSION_PRODUCTS_GA4,
            $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setProductAdditionalInfo($productInfo): Session
    {
        $items = $this->getProductAdditionalInfo();

        $this->storage->setData(self::KEY_PRODUCT_ADDITIONAL_INFO,
            $productInfo);

        return $this;
    }

    public function getProductAdditionalInfo(bool $clear = false): array
    {
        $items = $this->getData(self::KEY_PRODUCT_ADDITIONAL_INFO, $clear);

        if (null === $items) {
            return [];
        }

        if (!is_array($items)) {
            return [];
        }

        return 0 < count($items) ? $items : [];
    }

    public function setBehavioralEvent($behavioralEvent): Session
    {
        $events = $this->getBehavioralEvent();

        $this->storage->setData(self::KEY_BEHAVIORAL_EVENT,
            array_merge($events, [$behavioralEvent]));

        return $this;
    }

    public function getBehavioralEvent(bool $clear = false): array
    {
        $events = $this->getData(self::KEY_BEHAVIORAL_EVENT, $clear);

        if (null === $events) {
            return [];
        }

        if (!is_array($events)) {
            return [];
        }

        return 0 < count($events) ? $events : [];
    }
}
